@@ -35,6 +35,8 @@ struct dlm_lksb {
 /* dlm_new_lockspace flags */
 #define DLM_LSFL_NODIR          0x00000001
 #define DLM_LSFL_TIMEWARN       0x00000002
+#define DLM_LSFL_FS             0x00000004
+#define DLM_LSFL_NEWEXCL        0x00000008
 
 #endif
 
@@ -218,9 +220,9 @@ extern int dlm_ls_purge(dlm_lshandle_t lockspace,
  */
 
 #ifdef _REENTRANT
-extern int dlm_pthread_init();
+extern int dlm_pthread_init(void);
 extern int dlm_ls_pthread_init(dlm_lshandle_t lockspace);
-extern int dlm_pthread_cleanup();
+extern int dlm_pthread_cleanup(void);
 #endif
 
 
