@@ -1,11 +1,14 @@
 #include "fd.h"
 #include "config.h"
+#include <arpa/inet.h>
 #include <libcman.h>
 
 #define BUFLEN		MAX_NODENAME_LEN+1
 
 static cman_handle_t	ch;
 static cman_handle_t	ch_admin;
+static cman_node_t	old_nodes[MAX_NODES];
+static int		old_node_count;
 static cman_node_t	cman_nodes[MAX_NODES];
 static int		cman_node_count;
 
@@ -30,9 +33,31 @@ void kick_node_from_cluster(int nodeid)
 	}
 }
 
+static int is_member(cman_node_t *node_list, int count, int nodeid)
+{
+	int i;
+
+	for (i = 0; i < count; i++) {
+		if (node_list[i].cn_nodeid == nodeid)
+			return node_list[i].cn_member;
+	}
+	return 0;
+}
+
+static int is_old_member(int nodeid)
+{
+	return is_member(old_nodes, old_node_count, nodeid);
+}
+
+static int is_cluster_member(int nodeid)
+{
+	return is_member(cman_nodes, cman_node_count, nodeid);
+}
+
 static int name_equal(char *name1, char *name2)
 {
 	char name3[BUFLEN], name4[BUFLEN];
+	char addr1[INET6_ADDRSTRLEN];
 	int i, len1, len2;
 
 	len1 = strlen(name1);
@@ -41,6 +66,16 @@ static int name_equal(char *name1, char *name2)
 	if (len1 == len2 && !strncmp(name1, name2, len1))
 		return 1;
 
+	/*
+	 * If the names are IP addresses then don't compare
+	 * what is in front of the dots.
+	 */
+	if (inet_pton(AF_INET, name1, addr1) == 0)
+		return 0;
+
+	if (inet_pton(AF_INET6, name1, addr1) == 0)
+		return 0;
+
 	memset(name3, 0, BUFLEN);
 	memset(name4, 0, BUFLEN);
 
@@ -97,7 +132,7 @@ char *nodeid_to_name(int nodeid)
 	if (cn)
 		return cn->cn_name;
 
-	return "unknown";
+	return NULL;
 }
 
 int name_to_nodeid(char *name)
@@ -111,22 +146,70 @@ int name_to_nodeid(char *name)
 	return -1;
 }
 
-static void statechange(void)
+static void update_cluster(void)
+{
+	int quorate = cluster_quorate;
+	int i, rv;
+
+	cluster_quorate = cman_is_quorate(ch);
+
+	if (!quorate && cluster_quorate)
+		quorate_time = time(NULL);
+
+	old_node_count = cman_node_count;
+	memcpy(&old_nodes, &cman_nodes, sizeof(old_nodes));
+
+	cman_node_count = 0;
+	memset(&cman_nodes, 0, sizeof(cman_nodes));
+	rv = cman_get_nodes(ch, MAX_NODES, &cman_node_count, cman_nodes);
+	if (rv < 0) {
+		log_error("cman_get_nodes error %d %d", rv, errno);
+		return;
+	}
+
+	for (i = 0; i < old_node_count; i++) {
+		if (old_nodes[i].cn_member &&
+		    !is_cluster_member(old_nodes[i].cn_nodeid)) {
+
+			log_debug("cluster node %d removed",
+				  old_nodes[i].cn_nodeid);
+
+			node_history_cluster_remove(old_nodes[i].cn_nodeid);
+		}
+	}
+
+	for (i = 0; i < cman_node_count; i++) {
+		if (cman_nodes[i].cn_member &&
+		    !is_old_member(cman_nodes[i].cn_nodeid)) {
+
+			log_debug("cluster node %d added",
+				  cman_nodes[i].cn_nodeid);
+
+			node_history_cluster_add(cman_nodes[i].cn_nodeid);
+		}
+	}
+}
+
+/* Note: in fence delay loop we aren't processing callbacks so won't
+   have done an update_cluster() in response to a cman callback */
+
+int is_cluster_member_reread(int nodeid)
 {
 	int rv;
 
-	cman_quorate = cman_is_quorate(ch);
-	cman_node_count = 0;
-	memset(&cman_nodes, 0, sizeof(cman_nodes));
+	update_cluster();
 
-	rv = cman_get_nodes(ch, MAX_NODES, &cman_node_count, cman_nodes);
-	if (rv < 0)
-		log_error("cman_get_nodes error %d %d", rv, errno);
+	rv = is_cluster_member(nodeid);
+	if (rv)
+		return 1;
+
+	/* log_debug("cman_member %d not member", nodeid); */
+	return 0;
 }
 
 static void cman_callback(cman_handle_t h, void *private, int reason, int arg)
 {
-	int quorate = cman_quorate;
+	int quorate = cluster_quorate;
 
 	switch (reason) {
 	case CMAN_REASON_TRY_SHUTDOWN:
@@ -138,20 +221,21 @@ static void cman_callback(cman_handle_t h, void *private, int reason, int arg)
 		}
 		break;
 	case CMAN_REASON_STATECHANGE:
-		statechange();
+		update_cluster();
 
 		/* domain may have been waiting for quorum */
-		if (!quorate && cman_quorate && (group_mode == GROUP_LIBCPG))
+		if (!quorate && cluster_quorate && (group_mode == GROUP_LIBCPG))
 			process_fd_changes();
 		break;
 
 	case CMAN_REASON_CONFIG_UPDATE:
 		setup_logging();
+		reread_ccs();
 		break;
 	}
 }
 
-void process_cman(int ci)
+void process_cluster(int ci)
 {
 	int rv;
 
@@ -160,7 +244,7 @@ void process_cman(int ci)
 		cluster_dead(0);
 }
 
-int setup_cman(void)
+int setup_cluster(void)
 {
 	cman_node_t node;
 	int rv, fd;
@@ -180,6 +264,7 @@ int setup_cman(void)
 	ch = cman_init(NULL);
 	if (!ch) {
 		log_error("cman_init error %d", errno);
+		cman_finish(ch_admin);
 		return -ENOTCONN;
 	}
 
@@ -192,6 +277,7 @@ int setup_cman(void)
 		}
 		log_error("cman_is_active error %d", errno);
 		cman_finish(ch);
+		cman_finish(ch_admin);
 		return -ENOTCONN;
 	}
 
@@ -199,10 +285,11 @@ int setup_cman(void)
 	if (rv < 0) {
 		log_error("cman_start_notification error %d %d", rv, errno);
 		cman_finish(ch);
+		cman_finish(ch_admin);
 		return rv;
 	}
 
-	statechange();
+	update_cluster();
 
 	fd = cman_get_fd(ch);
 
@@ -212,6 +299,7 @@ int setup_cman(void)
 	if (rv < 0) {
 		log_error("cman_get_node us error %d %d", rv, errno);
 		cman_finish(ch);
+		cman_finish(ch_admin);
 		fd = rv;
 		goto out;
 	}
@@ -225,25 +313,10 @@ int setup_cman(void)
 	return fd;
 }
 
-void close_cman(void)
+void close_cluster(void)
 {
 	cman_finish(ch);
-}
-
-int is_cman_member(int nodeid)
-{
-	cman_node_t *cn;
-
-	/* Note: in fence delay loop we aren't processing callbacks so won't
-	   have done a statechange() in response to a cman callback */
-	statechange();
-
-	cn = find_cman_node(nodeid);
-	if (cn && cn->cn_member)
-		return 1;
-
-	log_debug("node %d not a cman member, cn %d", nodeid, cn ? 1 : 0);
-	return 0;
+	cman_finish(ch_admin);
 }
 
 struct node *get_new_node(struct fd *fd, int nodeid)
