@@ -153,6 +153,7 @@ static int make_dinode(struct fsck_inode *dip, struct gfs_inum *inum,
 	if(check_meta(dibh, 0)){
 		log_err("make_dinode:  Buffer #%"PRIu64" has no meta header.\n",
 			BH_BLKNO(dibh));
+		errors_found++;
 		if(query(dip->i_sbd, "Add header? (y/n) ")){
 			struct gfs_meta_header mh;
 			memset(&mh, 0, sizeof(struct gfs_meta_header));
@@ -160,6 +161,7 @@ static int make_dinode(struct fsck_inode *dip, struct gfs_inum *inum,
 			mh.mh_type = GFS_METATYPE_NONE;
 			gfs_meta_header_out(&mh, BH_DATA(dibh));
 			log_warn("meta header added.\n");
+			errors_corrected++;
 		} else {
 			log_err("meta header not added.  Failing make_dinode.\n");
 			relse_buf(sdp, dibh);
@@ -200,7 +202,7 @@ static int make_dinode(struct fsck_inode *dip, struct gfs_inum *inum,
 	rgd = fs_blk2rgrpd(sdp, inum->no_addr);
 	if(!rgd){
 		log_err( "Unable to map block #%"PRIu64" to rgrp\n", inum->no_addr);
-		exit(1);
+		exit(FSCK_ERROR);
 	}
 
 	di.di_rgrp = rgd->rd_ri.ri_addr;
@@ -307,7 +309,7 @@ static int fs_lookupi(struct fsck_inode *dip, osi_filename_t *name,
 	return error;
 }
 
-int fs_createi(struct fsck_inode *dip, osi_filename_t *name,
+static int fs_createi(struct fsck_inode *dip, osi_filename_t *name,
 	       unsigned int type, unsigned int mode, osi_cred_t *cred,
 	       int *new, struct fsck_inode **ipp)
 {
@@ -492,7 +494,7 @@ int fs_createi(struct fsck_inode *dip, osi_filename_t *name,
  *
  * Returns: 0 on success, -1 on failure.
  */
-int fs_mkdir(struct fsck_inode *dip, char *new_dir, int mode, struct fsck_inode **nip){
+int fs_mkdir(struct fsck_inode *dip, const char *new_dir, int mode, struct fsck_inode **nip){
 	int error;
 	osi_cred_t creds;
 	osi_buf_t *dibh;
@@ -501,9 +503,9 @@ int fs_mkdir(struct fsck_inode *dip, char *new_dir, int mode, struct fsck_inode
 	struct fsck_inode *ip= NULL;
 	struct fsck_sb *sdp = dip->i_sbd;
 	osi_filename_t name;
-	int new;
+	int new = 0;
 
-	name.name = new_dir;
+	name.name = (unsigned char *)new_dir;
 	name.len = strlen(new_dir);
 	creds.cr_uid = getuid();
 	creds.cr_gid = getgid();
