@@ -3,11 +3,17 @@
 #include <string.h>
 #include <stdint.h>
 #include <inttypes.h>
+#include <ctype.h>
 
 #include <linux/types.h>
 #include "libgfs2.h"
 
-#include "gfs2_disk_hash.h"
+#define pv(struct, member, fmt, fmt2) do {				\
+		print_it("  "#member, fmt, fmt2, struct->member);	\
+	} while (FALSE);
+#define pv2(struct, member, fmt, fmt2) do {				\
+		print_it("  ", fmt, fmt2, struct->member);		\
+	} while (FALSE);
 
 
 #define CPIN_08(s1, s2, member, count) {memcpy((s1->member), (s2->member), (count));}
@@ -76,7 +82,7 @@ void gfs2_meta_header_out(struct gfs2_meta_header *mh, char *buf)
 
 void gfs2_meta_header_print(struct gfs2_meta_header *mh)
 {
-	pv(mh, mh_magic, "0x%.8X", NULL);
+	pv(mh, mh_magic, "0x%08X", NULL);
 	pv(mh, mh_type, "%u", "0x%x");
 	pv(mh, mh_format, "%u", "0x%x");
 }
@@ -98,6 +104,9 @@ void gfs2_sb_in(struct gfs2_sb *sb, char *buf)
 
 	CPIN_08(sb, str, sb_lockproto, GFS2_LOCKNAME_LEN);
 	CPIN_08(sb, str, sb_locktable, GFS2_LOCKNAME_LEN);
+#ifdef GFS2_HAS_UUID
+	CPIN_08(sb, str, sb_uuid, sizeof(sb->sb_uuid));
+#endif
 }
 
 void gfs2_sb_out(struct gfs2_sb *sb, char *buf)
@@ -117,8 +126,37 @@ void gfs2_sb_out(struct gfs2_sb *sb, char *buf)
 
 	CPOUT_08(sb, str, sb_lockproto, GFS2_LOCKNAME_LEN);
 	CPOUT_08(sb, str, sb_locktable, GFS2_LOCKNAME_LEN);
+#ifdef GFS2_HAS_UUID
+	memcpy(str->sb_uuid, sb->sb_uuid, 16);
+#endif
 }
 
+const char *str_uuid(const unsigned char *uuid)
+{
+	static char str[64];
+	char *ch;
+	int i;
+
+	memset(str, 0, sizeof(str));
+	ch = str;
+	for (i = 0; i < 16; i++) {
+		sprintf(ch, "%02X", uuid[i]);
+		ch += 2;
+		if ((i == 3) || (i == 5) || (i == 7) || (i == 9)) {
+			*ch = '-';
+			ch++;
+		}
+	}
+	return str;
+}
+
+#ifdef GFS2_HAS_UUID
+void gfs2_print_uuid(const unsigned char *uuid)
+{
+	print_it("  uuid", "%s", NULL, str_uuid(uuid));
+}
+#endif
+
 void gfs2_sb_print(struct gfs2_sb *sb)
 {
 	gfs2_meta_header_print(&sb->sb_header);
@@ -134,6 +172,10 @@ void gfs2_sb_print(struct gfs2_sb *sb)
 
 	pv(sb, sb_lockproto, "%s", NULL);
 	pv(sb, sb_locktable, "%s", NULL);
+
+#ifdef GFS2_HAS_UUID
+	gfs2_print_uuid(sb->sb_uuid);
+#endif
 }
 
 void gfs2_rindex_in(struct gfs2_rindex *ri, char *buf)
@@ -217,8 +259,7 @@ void gfs2_quota_in(struct gfs2_quota *qu, char *buf)
 	CPIN_64(qu, str, qu_limit);
 	CPIN_64(qu, str, qu_warn);
 	CPIN_64(qu, str, qu_value);
-	CPIN_32(qu, str, qu_ll_next);
-	CPIN_08(qu, str, qu_reserved, 60);
+	CPIN_08(qu, str, qu_reserved, sizeof(qu->qu_reserved));
 }
 
 void gfs2_quota_out(struct gfs2_quota *qu, char *buf)
@@ -228,8 +269,7 @@ void gfs2_quota_out(struct gfs2_quota *qu, char *buf)
 	CPOUT_64(qu, str, qu_limit);
 	CPOUT_64(qu, str, qu_warn);
 	CPOUT_64(qu, str, qu_value);
-	CPOUT_32(qu, str, qu_ll_next);
-	CPOUT_08(qu, str, qu_reserved, 60);
+	memset(qu->qu_reserved, 0, sizeof(qu->qu_reserved));
 }
 
 void gfs2_quota_print(struct gfs2_quota *qu)
