@@ -12,7 +12,7 @@
 
 #include "libgfs2.h"
 
-void build_master(struct gfs2_sbd *sdp)
+int build_master(struct gfs2_sbd *sdp)
 {
 	struct gfs2_inum inum;
 	uint64_t bn;
@@ -30,10 +30,10 @@ void build_master(struct gfs2_sbd *sdp)
 		printf("\nMaster dir:\n");
 		gfs2_dinode_print(&sdp->master_dir->i_di);
 	}
+	return 0;
 }
 
-void
-build_sb(struct gfs2_sbd *sdp)
+void build_sb(struct gfs2_sbd *sdp, const unsigned char *uuid)
 {
 	unsigned int x;
 	struct gfs2_buffer_head *bh;
@@ -41,7 +41,7 @@ build_sb(struct gfs2_sbd *sdp)
 
 	/* Zero out the beginning of the device up to the superblock */
 	for (x = 0; x < sdp->sb_addr; x++) {
-		bh = bget(sdp, x);
+		bh = bget(&sdp->buf_list, x);
 		memset(bh->b_data, 0, sdp->bsize);
 		brelse(bh, updated);
 	}
@@ -59,17 +59,9 @@ build_sb(struct gfs2_sbd *sdp)
 	strcpy(sb.sb_lockproto, sdp->lockproto);
 	strcpy(sb.sb_locktable, sdp->locktable);
 #ifdef GFS2_HAS_UUID
-	{
-		int fd = open("/dev/urandom", O_RDONLY);
-		int n;
-		if (fd >= 0)
-			n = read(fd, &sb.sb_uuid, 16);
-		if (fd < 0 || n != 16)
-			memset(&sb.sb_uuid, 0, 16);
-		close(fd);
-	}
+	memcpy(sb.sb_uuid, uuid, sizeof(sb.sb_uuid));
 #endif
-	bh = bget(sdp, sdp->sb_addr);
+	bh = bget(&sdp->buf_list, sdp->sb_addr);
 	gfs2_sb_out(&sb, bh->b_data);
 	brelse(bh, updated);
 
@@ -79,12 +71,12 @@ build_sb(struct gfs2_sbd *sdp)
 	}
 }
 
-void write_journal(struct gfs2_sbd *sdp, struct gfs2_inode *ip, unsigned int j,
-				   unsigned int blocks)
+int write_journal(struct gfs2_sbd *sdp, struct gfs2_inode *ip, unsigned int j,
+		  unsigned int blocks)
 {
 	struct gfs2_log_header lh;
 	unsigned int x;
-	uint64_t seq = RANDOM(blocks);
+	uint64_t seq = ((blocks) * (random() / (RAND_MAX + 1.0)));
 	uint32_t hash;
 	unsigned int height;
 
@@ -102,13 +94,13 @@ void write_journal(struct gfs2_sbd *sdp, struct gfs2_inode *ip, unsigned int j,
 	for (x = 0; x < blocks; x++) {
 		struct gfs2_buffer_head *bh = get_file_buf(ip, x, TRUE);
 		if (!bh)
-			die("write_journal\n");
+			return -1;
 		brelse(bh, updated);
 	}
 	for (x = 0; x < blocks; x++) {
 		struct gfs2_buffer_head *bh = get_file_buf(ip, x, FALSE);
 		if (!bh)
-			die("write_journal\n");
+			return -1;
 
 		lh.lh_sequence = seq;
 		lh.lh_blkno = x;
@@ -126,13 +118,14 @@ void write_journal(struct gfs2_sbd *sdp, struct gfs2_inode *ip, unsigned int j,
 		printf("\nJournal %u:\n", j);
 		gfs2_dinode_print(&ip->i_di);
 	}
+	return 0;
 }
 
-void
-build_jindex(struct gfs2_sbd *sdp)
+int build_jindex(struct gfs2_sbd *sdp)
 {
 	struct gfs2_inode *jindex;
 	unsigned int j;
+	int ret;
 
 	jindex = createi(sdp->master_dir, "jindex", S_IFDIR | 0700,
 			 GFS2_DIF_SYSTEM);
@@ -143,8 +136,10 @@ build_jindex(struct gfs2_sbd *sdp)
 
 		sprintf(name, "journal%u", j);
 		ip = createi(jindex, name, S_IFREG | 0600, GFS2_DIF_SYSTEM);
-		write_journal(sdp, ip, j,
+		ret = write_journal(sdp, ip, j,
 			      sdp->jsize << 20 >> sdp->sd_sb.sb_bsize_shift);
+		if (ret)
+			return ret;
 		inode_put(ip, updated);
 	}
 
@@ -154,10 +149,10 @@ build_jindex(struct gfs2_sbd *sdp)
 	}
 
 	inode_put(jindex, updated);
+	return 0;
 }
 
-static void
-build_inum_range(struct gfs2_inode *per_node, unsigned int j)
+static int build_inum_range(struct gfs2_inode *per_node, unsigned int j)
 {
 	struct gfs2_sbd *sdp = per_node->i_sbd;
 	char name[256];
@@ -174,10 +169,10 @@ build_inum_range(struct gfs2_inode *per_node, unsigned int j)
 	}
 
 	inode_put(ip, updated);
+	return 0;
 }
 
-static void
-build_statfs_change(struct gfs2_inode *per_node, unsigned int j)
+static void build_statfs_change(struct gfs2_inode *per_node, unsigned int j)
 {
 	struct gfs2_sbd *sdp = per_node->i_sbd;
 	char name[256];
@@ -196,8 +191,7 @@ build_statfs_change(struct gfs2_inode *per_node, unsigned int j)
 	inode_put(ip, updated);
 }
 
-static void
-build_quota_change(struct gfs2_inode *per_node, unsigned int j)
+static int build_quota_change(struct gfs2_inode *per_node, unsigned int j)
 {
 	struct gfs2_sbd *sdp = per_node->i_sbd;
 	struct gfs2_meta_header mh;
@@ -218,7 +212,7 @@ build_quota_change(struct gfs2_inode *per_node, unsigned int j)
 	for (x = 0; x < blocks; x++) {
 		struct gfs2_buffer_head *bh = get_file_buf(ip, ip->i_di.di_size >> sdp->sd_sb.sb_bsize_shift, FALSE);
 		if (!bh)
-			die("build_quota_change\n");
+			return -1;
 
 		gfs2_meta_header_out(&mh, bh->b_data);
 
@@ -231,10 +225,10 @@ build_quota_change(struct gfs2_inode *per_node, unsigned int j)
 	}
 
 	inode_put(ip, updated);
+	return 0;
 }
 
-void
-build_per_node(struct gfs2_sbd *sdp)
+int build_per_node(struct gfs2_sbd *sdp)
 {
 	struct gfs2_inode *per_node;
 	unsigned int j;
@@ -254,10 +248,10 @@ build_per_node(struct gfs2_sbd *sdp)
 	}
 
 	inode_put(per_node, updated);
+	return 0;
 }
 
-void
-build_inum(struct gfs2_sbd *sdp)
+int build_inum(struct gfs2_sbd *sdp)
 {
 	struct gfs2_inode *ip;
 
@@ -270,10 +264,10 @@ build_inum(struct gfs2_sbd *sdp)
 	}
 
 	sdp->md.inum = ip;
+	return 0;
 }
 
-void
-build_statfs(struct gfs2_sbd *sdp)
+int build_statfs(struct gfs2_sbd *sdp)
 {
 	struct gfs2_inode *ip;
 
@@ -286,10 +280,10 @@ build_statfs(struct gfs2_sbd *sdp)
 	}
 
 	sdp->md.statfs = ip;
+	return 0;
 }
 
-void
-build_rindex(struct gfs2_sbd *sdp)
+int build_rindex(struct gfs2_sbd *sdp)
 {
 	struct gfs2_inode *ip;
 	osi_list_t *tmp, *head;
@@ -311,7 +305,7 @@ build_rindex(struct gfs2_sbd *sdp)
 		count = gfs2_writei(ip, buf, ip->i_di.di_size,
 							sizeof(struct gfs2_rindex));
 		if (count != sizeof(struct gfs2_rindex))
-			die("build_rindex\n");
+			return -1;
 	}
 
 	if (sdp->debug) {
@@ -320,10 +314,10 @@ build_rindex(struct gfs2_sbd *sdp)
 	}
 
 	inode_put(ip, updated);
+	return 0;
 }
 
-void
-build_quota(struct gfs2_sbd *sdp)
+int build_quota(struct gfs2_sbd *sdp)
 {
 	struct gfs2_inode *ip;
 	struct gfs2_quota qu;
@@ -340,10 +334,10 @@ build_quota(struct gfs2_sbd *sdp)
 
 	count = gfs2_writei(ip, buf, ip->i_di.di_size, sizeof(struct gfs2_quota));
 	if (count != sizeof(struct gfs2_quota))
-		die("do_init (2)\n");
+		return -1;
 	count = gfs2_writei(ip, buf, ip->i_di.di_size, sizeof(struct gfs2_quota));
 	if (count != sizeof(struct gfs2_quota))
-		die("do_init (3)\n");
+		return -1;
 
 	if (sdp->debug) {
 		printf("\nRoot quota:\n");
@@ -351,10 +345,10 @@ build_quota(struct gfs2_sbd *sdp)
 	}
 
 	inode_put(ip, updated);
+	return 0;
 }
 
-void
-build_root(struct gfs2_sbd *sdp)
+int build_root(struct gfs2_sbd *sdp)
 {
 	struct gfs2_inum inum;
 	uint64_t bn;
@@ -371,46 +365,47 @@ build_root(struct gfs2_sbd *sdp)
 		printf("\nRoot directory:\n");
 		gfs2_dinode_print(&sdp->md.rooti->i_di);
 	}
+	return 0;
 }
 
-void
-do_init(struct gfs2_sbd *sdp)
+int do_init_inum(struct gfs2_sbd *sdp)
 {
-	{
-		struct gfs2_inode *ip = sdp->md.inum;
-		uint64_t buf;
-		int count;
+	struct gfs2_inode *ip = sdp->md.inum;
+	uint64_t buf;
+	int count;
 
-		buf = cpu_to_be64(sdp->md.next_inum);
-		count = gfs2_writei(ip, &buf, 0, sizeof(uint64_t));
-		if (count != sizeof(uint64_t))
-			die("do_init (1)\n");
+	buf = cpu_to_be64(sdp->md.next_inum);
+	count = gfs2_writei(ip, &buf, 0, sizeof(uint64_t));
+	if (count != sizeof(uint64_t))
+		return -1;
 
-		if (sdp->debug)
-			printf("\nNext Inum: %"PRIu64"\n",
-			       sdp->md.next_inum);
-	}
+	if (sdp->debug)
+		printf("\nNext Inum: %"PRIu64"\n",
+		       sdp->md.next_inum);
+	return 0;
+}
 
-	{
-		struct gfs2_inode *ip = sdp->md.statfs;
-		struct gfs2_statfs_change sc;
-		char buf[sizeof(struct gfs2_statfs_change)];
-		int count;
+int do_init_statfs(struct gfs2_sbd *sdp)
+{
+	struct gfs2_inode *ip = sdp->md.statfs;
+	struct gfs2_statfs_change sc;
+	char buf[sizeof(struct gfs2_statfs_change)];
+	int count;
 
-		sc.sc_total = sdp->blks_total;
-		sc.sc_free = sdp->blks_total - sdp->blks_alloced;
-		sc.sc_dinodes = sdp->dinodes_alloced;
+	sc.sc_total = sdp->blks_total;
+	sc.sc_free = sdp->blks_total - sdp->blks_alloced;
+	sc.sc_dinodes = sdp->dinodes_alloced;
 
-		gfs2_statfs_change_out(&sc, buf);
-		count = gfs2_writei(ip, buf, 0, sizeof(struct gfs2_statfs_change));
-		if (count != sizeof(struct gfs2_statfs_change))
-			die("do_init (2)\n");
+	gfs2_statfs_change_out(&sc, buf);
+	count = gfs2_writei(ip, buf, 0, sizeof(struct gfs2_statfs_change));
+	if (count != sizeof(struct gfs2_statfs_change))
+		return -1;
 
-		if (sdp->debug) {
-			printf("\nStatfs:\n");
-			gfs2_statfs_change_print(&sc);
-		}
+	if (sdp->debug) {
+		printf("\nStatfs:\n");
+		gfs2_statfs_change_print(&sc);
 	}
+	return 0;
 }
 
 struct gfs2_inode *gfs2_load_inode(struct gfs2_sbd *sbp, uint64_t block)
@@ -418,7 +413,7 @@ struct gfs2_inode *gfs2_load_inode(struct gfs2_sbd *sbp, uint64_t block)
 	struct gfs2_buffer_head *bh;
 	struct gfs2_inode *ip;
 
-	bh = bread(sbp, block);
+	bh = bread(&sbp->buf_list, block);
 	ip = inode_get(sbp, bh);
 	return ip;
 }
@@ -435,32 +430,6 @@ int gfs2_check_meta(struct gfs2_buffer_head *bh, int type)
 	return 0;
 }
 
-/*
- * set_meta - set the meta header of a buffer
- * @bh
- * @type
- *
- * Returns: 0 if ok, -1 on error
- */
-int gfs2_set_meta(struct gfs2_buffer_head *bh, int type, int format)
-{
-	struct gfs2_meta_header header;
-
-	if(!gfs2_check_meta(bh, 0)){
-		((struct gfs2_meta_header *)bh->b_data)->mh_type = cpu_to_be32(type);
-		((struct gfs2_meta_header *)bh->b_data)->mh_format = 
-			cpu_to_be32(format);
-	} else {
-		memset(&header, 0, sizeof(struct gfs2_meta_header));
-		header.mh_magic = GFS2_MAGIC;
-		header.mh_type = type;
-		header.mh_format = format;
-		
-		gfs2_meta_header_out(&header, bh->b_data);
-	}
-	return 0;
-}
-
 /**
  * gfs2_next_rg_meta
  * @rgd:
@@ -505,71 +474,6 @@ int gfs2_next_rg_meta(struct rgrp_list *rgd, uint64_t *block, int first)
 	return 0;
 }
 
-/**
- * gfs2_next_rg_meta_free - finds free or used metadata
- * @rgd:
- * @block:
- * @first: if set, start at zero and ignore block
- *
- * The position to start looking from is *block.  When a block
- * is found, it is returned in block.
- *
- * Returns: 0 on success, -1 when finished
- */
-int gfs2_next_rg_meta_free(struct rgrp_list *rgd, uint64_t *block, int first,
-						   int *mfree)
-{
-	gfs2_bitmap_t *bits = NULL;
-	uint32_t length = rgd->ri.ri_length;
-	uint32_t blk = (first)? 0: (uint32_t)((*block+1)-rgd->ri.ri_data0);
-	uint32_t iblk, ublk, fblk;
-	int i;
-	
-	if(!first && (*block < rgd->ri.ri_data0)) {
-		log_err("next_rg_meta_free:  Start block is outside rgrp bounds.\n");
-		exit(1);
-	}
-	for(i=0; i < length; i++){
-		bits = &rgd->bits[i];
-		if(blk < bits->bi_len*GFS2_NBBY)
-			break;
-		blk -= bits->bi_len*GFS2_NBBY;
-	}
-	for(; i < length; i++){
-		bits = &rgd->bits[i];
-
-		iblk = gfs2_bitfit((unsigned char *)rgd->bh[i]->b_data +
-						   bits->bi_offset, bits->bi_len, blk,
-						   GFS2_BLKST_DINODE);
-		ublk = gfs2_bitfit((unsigned char *)rgd->bh[i]->b_data +
-						   bits->bi_offset, bits->bi_len, blk,
-						   GFS2_BLKST_USED);
-		fblk = gfs2_bitfit((unsigned char *)rgd->bh[i]->b_data +
-						   bits->bi_offset, bits->bi_len, blk,
-						   GFS2_BLKST_FREE);
-		if(ublk < fblk) {
-            blk = ublk;
-            *mfree = 0;
-		}
-		else if(iblk < fblk) {
-            blk = iblk;
-            *mfree = 0;
-		} else {
-            blk = fblk;
-            *mfree = 1;
-		}
-		if(blk != BFITNOENT){
-            *block = blk + (bits->bi_start * GFS2_NBBY) + rgd->ri.ri_data0;
-            break;
-		}
-		blk=0;
-	}
-
-	if(i == length)
-		return -1;
-	return 0;
-}
-
 /**
  * next_rg_metatype
  * @rgd:
@@ -580,7 +484,7 @@ int gfs2_next_rg_meta_free(struct rgrp_list *rgd, uint64_t *block, int first,
  * Returns: 0 on success, -1 on error or finished
  */
 int gfs2_next_rg_metatype(struct gfs2_sbd *sdp, struct rgrp_list *rgd,
-						  uint64_t *block, uint32_t type, int first)
+			  uint64_t *block, uint32_t type, int first)
 {
 	struct gfs2_buffer_head *bh = NULL;
 
@@ -589,7 +493,7 @@ int gfs2_next_rg_metatype(struct gfs2_sbd *sdp, struct rgrp_list *rgd,
 			brelse(bh, not_updated);
 		if (gfs2_next_rg_meta(rgd, block, first))
 			return -1;
-		bh = bread(sdp, *block);
+		bh = bread(&sdp->buf_list, *block);
 		first = 0;
 	} while(gfs2_check_meta(bh, type));
 	brelse(bh, not_updated);
