@@ -18,6 +18,7 @@
 #include <inttypes.h>
 
 #include <linux/types.h>
+#include <linux/fiemap.h>
 #include "gfs2_quota.h"
 
 #define __user
@@ -28,6 +29,7 @@
 /*  Constants  */
 
 #define OPTION_STRING ("bdf:g:hkl:mnsu:V")
+#define FS_IOC_FIEMAP                   _IOWR('f', 11, struct fiemap)
 
 /**
  * This function is for libgfs2's sake.
@@ -348,139 +350,6 @@ write_quota_internal(int fd, uint32_t id, int id_type, struct gfs2_quota *q)
 		die("failed to write to quota file: %s\n", strerror(errno));
 }
 
-/**
- * get_last_quota_id - Get the last quota in the quota file
- * @fd: an open file descriptor of the quota file
- * @id_type: GQ_ID_USER or GQ_ID_GROUP
- * @max_id: return the maximum id obtained
- */
-void 
-get_last_quota_id(int fd, uint32_t *max_id)
-{
-	/* stat(2) the quota file to find how big it is. This will give us a
-	 * a rough idea of what the last valid quota uid/gid is. It is possible 
-	 * that the last quota in the file belongs to a group with gid:x and 
-	 * the corresponding user quota with uid:x doesn't exist. In such cases
-	 * we still return x as max_id. This max_id is ONLY A HINT. If used 
-	 * as a terminating condition of a loop, another condition should also
-	 * be specified.
-	 */
-	struct stat st;
-	uint32_t qsize = sizeof(struct gfs2_quota);
-	uint64_t size;
-	if (fstat(fd, &st))
-		die("failed to stat the quota file: %s\n", strerror(errno));
-	size = st.st_size;
-	if (!size)
-		die("error: quota file is truncated to zero!\n");
-	if (size % qsize) {
-		printf("warning: quota file size not a multiple of "
-		       "struct gfs2_quota\n");
-		size = qsize * (size / qsize);
-	}
-	*max_id = (size - 1) / (2 * qsize);
-}
-
-/**
- * is_valid_quota_list - Check if we have a valid quota list
- * @fd: an open file descriptor of the quota file
- * Returns 0 or 1.
- */
-int 
-is_valid_quota_list(int fd)
-{
-	/* This is a slow test to determine if the quotas are in a 
-	 * linked list. We should come up with something better
-	 * Quota linked list format is identified by the following.
-	 * step1: Get the maximum groupid and userid having valid
-	 *        quotas in the quota file.
-	 * step2: Obtain the size of the quota file. The size of the 
-	 *        quota file (position of the last valid quota) 
-	 *        determines the last user/group id.
-	 * step3: If we can obtain the last valid quota through the 
-	 *        lists, then our lists are good. Else, the lists are 
-	 *        either corrupt or an older quota file format is in use
-	 */
-	int id_type = GQ_ID_GROUP;
-	uint32_t id = 0, prev, ulast = 0, glast = 0, max;
-	struct gfs2_quota q;
-
-	get_last_quota_id(fd, &max);
-again:
-	do {
-		read_quota_internal(fd, id, id_type, &q);
-		prev = id;
-		id = q.qu_ll_next;
-		if (id > max)
-			return 0;
-	} while (id && id > prev);
-
-	if (id && id <= prev)
-		return 0;
-
-	if (id_type == GQ_ID_GROUP)
-		glast = prev;
-	else
-		ulast = prev;
-
-	if (id_type == GQ_ID_GROUP) {
-		id_type = GQ_ID_USER;
-		id = 0;
-		goto again;
-	}
-
-	if (glast != max && ulast != max)
-		return 0;
-	
-	return 1;
-}
-
-void 
-print_quota_list_warning()
-{
-	printf("\nWarning: This filesystem doesn't seem to have the new quota "
-	       "list format or the quota list is corrupt. list, check and init "
-	       "operation performance will suffer due to this. It is recommended "
-	       "that you run the 'gfs2_quota reset' operation to reset the quota "
-	       "file. All current quota information will be lost and you will "
-	       "have to reassign all quota limits and warnings\n\n"); 
-}
-
-/**
- * adjust_quota_list - Adjust the quota linked list
- * @fd: The quota file descriptor
- * @comline: the struct containing the parsed command line arguments
- */
-static void
-adjust_quota_list(int fd, commandline_t *comline)
-{
-	uint32_t prev = 0, next = 0, id = comline->id;
-	struct gfs2_quota tmpq, q;
-	int id_type = comline->id_type;
-	
-	if (id == 0) /* root quota, don't do anything */
-		goto out;
-	/* We just wrote the quota for id in do_set(). Get it */
-	next = 0;
-	do {
-		read_quota_internal(fd, next, id_type, &q);
-		prev = next;
-		next = q.qu_ll_next;
-		if (prev == id) /* no duplicates, bail */
-			goto out;
-		if (prev < id && id < next) /* gotcha! */
-			break;
-	} while(next && next > prev);
-	read_quota_internal(fd, id, id_type, &tmpq);
-	tmpq.qu_ll_next = next;
-	q.qu_ll_next = id;
-	write_quota_internal(fd, id, id_type, &tmpq);
-	write_quota_internal(fd, prev, id_type, &q);
-
-out:
-	return;
-}
-
 /**
  * do_reset - Reset all the quota data for a filesystem
  * @comline: the struct containing the parsed command line arguments
@@ -531,11 +400,9 @@ do_reset(struct gfs2_sbd *sdp, commandline_t *comline)
 	}
 
 	read_quota_internal(fd, 0, GQ_ID_USER, &q);
-	q.qu_ll_next = 0;
 	write_quota_internal(fd, 0, GQ_ID_USER, &q);
 
 	read_quota_internal(fd, 0, GQ_ID_GROUP, &q);
-	q.qu_ll_next = 0;
 	write_quota_internal(fd, 0, GQ_ID_GROUP, &q);
 
 	/* truncate the quota file such that only the first
@@ -560,13 +427,14 @@ do_list(struct gfs2_sbd *sdp, commandline_t *comline)
 {
 	int fd;
 	struct gfs2_quota q;
-	char buf[sizeof(struct gfs2_quota)];
-	uint64_t offset;
-	uint32_t id, prev, maxid;
+	uint32_t startid, endid;
+	int id_type;
 	int pass = 0;
 	int error = 0;
 	char quota_file[BUF_SIZE];
-	int id_type = comline->id_type;
+	uint64_t quota_file_size = 0;
+	struct fiemap fmap = { 0, }, *fmap2;
+	struct stat statbuf;
 	
 	if (!*comline->filesystem)
 		die("need a filesystem to work on\n");
@@ -597,54 +465,68 @@ do_list(struct gfs2_sbd *sdp, commandline_t *comline)
 		die("can't open file %s: %s\n", quota_file,
 		    strerror(errno));
 	}
+	if (fstat(fd, &statbuf) < 0) {
+		close(fd);
+		close(sdp->metafs_fd);
+		cleanup_metafs(sdp);
+		die("can't stat file %s: %s\n", quota_file,
+		    strerror(errno));
+	}
+	quota_file_size = statbuf.st_size;
+	/* First find the number of extents in the quota file */
+	fmap.fm_start = 0;
+	fmap.fm_length = (~0ULL);
+	error = ioctl(fd, FS_IOC_FIEMAP, &fmap);
+	if (error == -1) {
+		fprintf(stderr, "fiemap error (%d): %s\n", errno, strerror(errno));
+		goto out;
+	}
+	fmap2 = malloc(sizeof(struct fiemap) + 
+		       fmap.fm_mapped_extents * sizeof(struct fiemap_extent));
+	if (fmap2 == NULL) {
+		fprintf(stderr, "malloc error (%d): %s\n", errno, strerror(errno));
+		goto out;
+	}
+	fmap2->fm_start = 0;
+	fmap2->fm_length = (~0ULL);
+	fmap2->fm_extent_count = fmap.fm_mapped_extents;
 	
-	if (!is_valid_quota_list(fd)) {
-		print_quota_list_warning();
-		goto do_old_school;
+	error = ioctl(fd, FS_IOC_FIEMAP, fmap2);
+	if (error == -1) {
+		fprintf(stderr, "fiemap error (%d): %s\n", errno, strerror(errno));
+		goto fmap2_free;
 	}
-	get_last_quota_id(fd, &maxid);
-	
-	for (pass=0; pass<2; pass++) {
-		id = 0;
-		id_type = pass ? GQ_ID_GROUP : GQ_ID_USER;
-		
-		do {
-			read_quota_internal(fd, id, id_type, &q);
-			prev = id;
-			if (q.qu_limit || q.qu_warn || q.qu_value)
-				print_quota(comline, 
-					    id_type == GQ_ID_USER ? TRUE : FALSE, 
-					    id, &q, &sdp->sd_sb);
-			id = q.qu_ll_next;
-		} while(id && id > prev && id <= maxid);
+	if (fmap2->fm_mapped_extents) {
+		int i;
+	again:
+		for (i=0; i<fmap2->fm_mapped_extents; i++) {
+			struct fiemap_extent *fe = &fmap2->fm_extents[i];
+			uint64_t end = fe->fe_logical + fe->fe_length;
+
+			end = end > quota_file_size ? quota_file_size : end;
+			startid = DIV_RU(fe->fe_logical, sizeof(struct gfs2_quota));
+			endid = end /(2 * sizeof(struct gfs2_quota));
+			if (startid % 2 != pass)
+				startid++;
+
+			startid = DIV_RU(startid, 2);
+			id_type = pass == 0 ? GQ_ID_USER : GQ_ID_GROUP;
+			do {
+				read_quota_internal(fd, startid, id_type, &q);
+				if (q.qu_limit || q.qu_warn || q.qu_value)
+					print_quota(comline, (pass) ? FALSE : TRUE, startid,
+						    &q, &sdp->sd_sb);
+				startid++;
+			} while (startid < endid);
+		}
+		if (!pass) {
+			pass = 1;
+			goto again;
+		}
 	}
-	goto out;
 
-do_old_school:
-	for (pass=0; pass<2; pass++) {
-		if (!pass)
-			offset = 0;
-		else
-			offset = sizeof(struct gfs2_quota);
-
-		do {
-			memset(buf, 0, sizeof(struct gfs2_quota));
-
-			/* read hidden quota file here */
-			lseek(fd, offset, SEEK_SET);
-			error = read(fd, buf, sizeof(struct gfs2_quota));
-
-			gfs2_quota_in(&q, buf);
-
-			id = (offset / sizeof(struct gfs2_quota)) >> 1;
-
-			if (q.qu_limit || q.qu_warn || q.qu_value)
-				print_quota(comline, (pass) ? FALSE : TRUE, id,
-					    &q, &sdp->sd_sb);
-
-			offset += 2 * sizeof(struct gfs2_quota);
-		} while (error == sizeof(struct gfs2_quota));
-	}
+fmap2_free:
+	free(fmap2);
 out:
 	close(fd);
 	close(sdp->metafs_fd);
@@ -666,7 +548,6 @@ do_get_one(struct gfs2_sbd *sdp, commandline_t *comline, char *filesystem)
 	struct gfs2_quota q;
 	uint64_t offset;
 	int error;
-	uint32_t maxid;
 	char quota_file[BUF_SIZE];
 
 	strcpy(sdp->path_name, filesystem);
@@ -703,10 +584,6 @@ do_get_one(struct gfs2_sbd *sdp, commandline_t *comline, char *filesystem)
 
 	memset(&q, 0, sizeof(struct gfs2_quota));
 	
-	get_last_quota_id(fd, &maxid);
-	if (comline->id > maxid)
-		goto print_empty_quota;
-
 	lseek(fd, offset, SEEK_SET);
 	error = read(fd, buf, sizeof(struct gfs2_quota));
 	if (error < 0) {
@@ -718,9 +595,6 @@ do_get_one(struct gfs2_sbd *sdp, commandline_t *comline, char *filesystem)
 	}
 
 	gfs2_quota_in(&q, buf);
-
-
-print_empty_quota:
 	print_quota(comline,
 		    (comline->id_type == GQ_ID_USER), comline->id,
 		    &q, &sdp->sd_sb);
@@ -839,7 +713,7 @@ do_set(struct gfs2_sbd *sdp, commandline_t *comline)
 	int fd;
 	uint64_t offset;
 	uint64_t new_value;
-	int error, adj_flag = 0;
+	int error;
 	char quota_file[BUF_SIZE];
 	char id_str[16];
 	struct stat stat_buf;
@@ -869,7 +743,7 @@ do_set(struct gfs2_sbd *sdp, commandline_t *comline)
 	strcpy(quota_file, sdp->metafs_path);
 	strcat(quota_file, "/quota");
 
-	fd = open(quota_file, O_RDWR);
+	fd = open(quota_file, O_WRONLY);
 	if (fd < 0) {
 		close(sdp->metafs_fd);
 		cleanup_metafs(sdp);
@@ -877,11 +751,6 @@ do_set(struct gfs2_sbd *sdp, commandline_t *comline)
 		    strerror(errno));
 	}
 	
-	if (is_valid_quota_list(fd))
-		adj_flag = 1;
-	else
-		print_quota_list_warning();
-
 	switch (comline->id_type) {
 	case GQ_ID_USER:
 		offset = (2 * (uint64_t)comline->id) * sizeof(struct gfs2_quota);
@@ -1008,8 +877,6 @@ do_set(struct gfs2_sbd *sdp, commandline_t *comline)
 		exit(-1);
 	}
 	
-	if (adj_flag)
-		adjust_quota_list(fd, comline);
 out:
 	close(fd);
 	close(sdp->metafs_fd);
