@@ -25,12 +25,12 @@
 #include "config.h"
 #include "ccs.h"
 
-static int ccs_handle;
+int ccs_handle;
 
 /* was a config value set on command line?, 0 or 1. */
 
 int optd_groupd_compat;
-int optd_debug_logsys;
+int optd_debug_logfile;
 int optd_enable_withdraw;
 int optd_enable_plock;
 int optd_plock_debug;
@@ -43,7 +43,7 @@ int optd_drop_resources_age;
 /* actual config value from command line, cluster.conf, or default. */
 
 int cfgd_groupd_compat		= DEFAULT_GROUPD_COMPAT;
-int cfgd_debug_logsys		= DEFAULT_DEBUG_LOGSYS;
+int cfgd_debug_logfile		= DEFAULT_DEBUG_LOGFILE;
 int cfgd_enable_withdraw	= DEFAULT_ENABLE_WITHDRAW;
 int cfgd_enable_plock		= DEFAULT_ENABLE_PLOCK;
 int cfgd_plock_debug		= DEFAULT_PLOCK_DEBUG;
@@ -53,7 +53,7 @@ int cfgd_drop_resources_time	= DEFAULT_DROP_RESOURCES_TIME;
 int cfgd_drop_resources_count	= DEFAULT_DROP_RESOURCES_COUNT;
 int cfgd_drop_resources_age	= DEFAULT_DROP_RESOURCES_AGE;
 
-void read_ccs_name(char *path, char *name)
+void read_ccs_name(const char *path, char *name)
 {
 	char *str;
 	int error;
@@ -67,7 +67,7 @@ void read_ccs_name(char *path, char *name)
 	free(str);
 }
 
-void read_ccs_yesno(char *path, int *yes, int *no)
+void read_ccs_yesno(const char *path, int *yes, int *no)
 {
 	char *str;
 	int error;
@@ -88,7 +88,7 @@ void read_ccs_yesno(char *path, int *yes, int *no)
 	free(str);
 }
 
-void read_ccs_int(char *path, int *config_val)
+int read_ccs_int(const char *path, int *config_val)
 {
 	char *str;
 	int val;
@@ -96,18 +96,19 @@ void read_ccs_int(char *path, int *config_val)
 
 	error = ccs_get(ccs_handle, path, &str);
 	if (error || !str)
-		return;
+		return -1;
 
 	val = atoi(str);
 
 	if (val < 0) {
 		log_error("ignore invalid value %d for %s", val, path);
-		return;
+		return -1;
 	}
 
 	*config_val = val;
 	log_debug("%s is %u", path, val);
 	free(str);
+	return 0;
 }
 
 #define LOCKSPACE_NODIR "/cluster/dlm/lockspace[@name=\"%s\"]/@nodir"
@@ -149,19 +150,32 @@ void read_ccs_nodir(struct mountgroup *mg, char *buf)
 #define DROP_RESOURCES_COUNT_PATH "/cluster/gfs_controld/@drop_resources_count"
 #define DROP_RESOURCES_AGE_PATH "/cluster/gfs_controld/@drop_resources_age"
 
+#define DLM_PLOCK_RATE_LIMIT_PATH "/cluster/dlm/@plock_rate_limit"
+#define DLM_PLOCK_OWNERSHIP_PATH "/cluster/dlm/@plock_ownership"
+#define DLM_DROP_RESOURCES_TIME_PATH "/cluster/dlm/@drop_resources_time"
+#define DLM_DROP_RESOURCES_COUNT_PATH "/cluster/dlm/@drop_resources_count"
+#define DLM_DROP_RESOURCES_AGE_PATH "/cluster/dlm/@drop_resources_age"
+
 int setup_ccs(void)
 {
-	int i = 0, cd;
+	int cd, rv;
 
-	while ((cd = ccs_connect()) < 0) {
-		sleep(1);
-		if (++i > 9 && !(i % 10))
-			log_error("connect to ccs error %d, "
-				  "check cluster status", cd);
+	if (ccs_handle)
+		goto update;
+
+	cd = ccs_connect();
+	if (cd < 0) {
+		log_error("ccs_connect error %d %d", cd, errno);
+		return -1;
 	}
-
 	ccs_handle = cd;
 
+	/* plock_ownership is a special case */
+	if (optd_plock_ownership)
+		using_default_plock_ownership = 0;
+	else
+		using_default_plock_ownership = 1;
+
 	/* These config values are set from cluster.conf only if they haven't
 	   already been set on the command line. */
 
@@ -171,18 +185,39 @@ int setup_ccs(void)
 		read_ccs_int(ENABLE_WITHDRAW_PATH, &cfgd_enable_withdraw);
 	if (!optd_enable_plock)
 		read_ccs_int(ENABLE_PLOCK_PATH, &cfgd_enable_plock);
-	if (!optd_plock_debug)
+	if (!optd_plock_ownership) {
+		rv = read_ccs_int(PLOCK_OWNERSHIP_PATH, &cfgd_plock_ownership);
+		if (rv < 0)
+			rv = read_ccs_int(DLM_PLOCK_OWNERSHIP_PATH, &cfgd_plock_ownership);
+		if (!rv)
+			using_default_plock_ownership = 0;
+	}
+
+	/* The following can be changed while running */
+ update:
+	if (!optd_plock_debug) {
 		read_ccs_int(PLOCK_DEBUG_PATH, &cfgd_plock_debug);
-	if (!optd_plock_rate_limit)
-		read_ccs_int(PLOCK_RATE_LIMIT_PATH, &cfgd_plock_rate_limit);
-	if (!optd_plock_ownership)
-		read_ccs_int(PLOCK_OWNERSHIP_PATH, &cfgd_plock_ownership);
-	if (!optd_drop_resources_time)
-		read_ccs_int(DROP_RESOURCES_TIME_PATH, &cfgd_drop_resources_time);
-	if (!optd_drop_resources_count)
-		read_ccs_int(DROP_RESOURCES_COUNT_PATH, &cfgd_drop_resources_count);
-	if (!optd_drop_resources_age)
-		read_ccs_int(DROP_RESOURCES_AGE_PATH, &cfgd_drop_resources_age);
+	}
+	if (!optd_plock_rate_limit) {
+		rv = read_ccs_int(PLOCK_RATE_LIMIT_PATH, &cfgd_plock_rate_limit);
+		if (rv < 0)
+			read_ccs_int(DLM_PLOCK_RATE_LIMIT_PATH, &cfgd_plock_rate_limit);
+	}
+	if (!optd_drop_resources_time) {
+		rv = read_ccs_int(DROP_RESOURCES_TIME_PATH, &cfgd_drop_resources_time);
+		if (rv < 0)
+			read_ccs_int(DLM_DROP_RESOURCES_TIME_PATH, &cfgd_drop_resources_time);
+	}
+	if (!optd_drop_resources_count) {
+		rv = read_ccs_int(DROP_RESOURCES_COUNT_PATH, &cfgd_drop_resources_count);
+		if (rv < 0)
+			read_ccs_int(DLM_DROP_RESOURCES_COUNT_PATH, &cfgd_drop_resources_count);
+	}
+	if (!optd_drop_resources_age) {
+		rv = read_ccs_int(DROP_RESOURCES_AGE_PATH, &cfgd_drop_resources_age);
+		if (rv < 0)
+			read_ccs_int(DLM_DROP_RESOURCES_AGE_PATH, &cfgd_drop_resources_age);
+	}
 
 	return 0;
 }
