@@ -36,7 +36,7 @@ typedef struct {
 	group_deliver_t deliver;
 } group_callbacks_t;
 
-group_handle_t group_init(void *priv, char *prog_name, int level, group_callbacks_t *cbs, int timeout);
+group_handle_t group_init(void *priv, const char *prog_name, int level, group_callbacks_t *cbs, int timeout);
 int group_exit(group_handle_t handle);
 
 int group_join(group_handle_t handle, char *name);
@@ -72,6 +72,8 @@ typedef struct group_data {
 int group_get_groups(int max, int *count, group_data_t *data);
 int group_get_group(int level, const char *name, group_data_t *data);
 
+int group_get_version(int *version);
+
 #ifdef __cplusplus
 }
 #endif
