@@ -1,28 +1,10 @@
-/*
-  Copyright Red Hat, Inc. 2006
-
-  This program is free software; you can redistribute it and/or modify it
-  under the terms of the GNU General Public License as published by the
-  Free Software Foundation; either version 2, or (at your option) any
-  later version.
-
-  This program is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  General Public License for more details.
-
-  You should have received a copy of the GNU General Public License
-  along with this program; see the file COPYING.  If not, write to the
-  Free Software Foundation, Inc.,  675 Mass Ave, Cambridge, 
-  MA 02139, USA.
-*/
 #define _MESSAGE_BUILD
-#include <message.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
 #include <string.h>
 #include <sys/types.h>
+#include <message.h>
 #include <stdlib.h>
 #include <sys/socket.h>
 #include <sys/stat.h>
@@ -199,7 +181,7 @@ sock_msg_fd_isset(msgctx_t *ctx, fd_set *fds)
 }
 
 
-int
+static int
 sock_msg_fd_clr(msgctx_t *ctx, fd_set *fds)
 {
 	errno = EINVAL;
@@ -286,7 +268,7 @@ sock_msg_receive(msgctx_t *ctx, void *msg, size_t maxlen, int timeout)
   If the speficied node is 0, this connects via the socket in
   /var/run/cluster...
  */
-int
+static int
 sock_msg_open(int type, int nodeid, int port, msgctx_t *ctx, int timeout)
 {
 	errno = EINVAL;
@@ -307,7 +289,7 @@ sock_msg_open(int type, int nodeid, int port, msgctx_t *ctx, int timeout)
 /**
   With a socket, the O/S cleans up the buffers for us.
  */
-int
+static int
 sock_msg_close(msgctx_t *ctx)
 {
 	errno = EINVAL;
@@ -329,6 +311,9 @@ sock_msg_close(msgctx_t *ctx)
 static int
 sock_msg_accept(msgctx_t *listenctx, msgctx_t *acceptctx)
 {
+	struct ucred cred;
+	socklen_t credlen = sizeof(cred);
+
 	errno = EINVAL;
 
 	if (!listenctx || !acceptctx)
@@ -345,6 +330,18 @@ sock_msg_accept(msgctx_t *listenctx, msgctx_t *acceptctx)
 	if (acceptctx->u.local_info.sockfd < 0)
 		return -1;
 
+	memset(&cred, 0, sizeof(cred));
+	if (getsockopt(acceptctx->u.local_info.sockfd, SOL_SOCKET,
+		       SO_PEERCRED, (void *)&cred, &credlen) < 0) {
+		perror("getsockopt");
+		cred.uid = (uid_t)-1;
+		cred.gid = (gid_t)-1;
+		cred.pid = (pid_t)-1;
+	}
+
+	memcpy(&acceptctx->u.local_info.cred, &cred,
+	       sizeof(cred));
+
 	set_cloexec(acceptctx->u.local_info.sockfd);
 
 	acceptctx->flags = (SKF_READ | SKF_WRITE);
@@ -353,7 +350,7 @@ sock_msg_accept(msgctx_t *listenctx, msgctx_t *acceptctx)
 
 
 int
-sock_msg_listen(int me, void *portp, msgctx_t **listen_ctx)
+sock_msg_listen(int me, const void *portp, msgctx_t **listen_ctx)
 {
 	int sock;
 	struct sockaddr_un su;
@@ -372,7 +369,7 @@ sock_msg_listen(int me, void *portp, msgctx_t **listen_ctx)
 
 	set_cloexec(sock);
 	unlink(RGMGR_SOCK);
-	om = umask(077);
+	om = umask(0117);
 	su.sun_family = PF_LOCAL;
 	snprintf(su.sun_path, sizeof(su.sun_path), "%s", path);
 
@@ -412,7 +409,7 @@ sock_msg_init(msgctx_t *ctx)
 }
 
 
-void
+static void
 sock_msg_print(msgctx_t *ctx)
 {
 	printf("Socket Message Context; fd = %d\n", ctx->u.local_info.sockfd);
