@@ -14,6 +14,7 @@
 #include <msgsimple.h>
 #include <vf.h>
 #include <lock.h>
+#include <sys/socket.h>
 #include <message.h>
 #include <rg_queue.h>
 #include <malloc.h>
@@ -34,6 +35,7 @@ void flag_shutdown(int sig);
 int watchdog_init(void);
 
 
+int cluster_timeout = 10;
 int shutdown_pending = 0, running = 1, need_reconfigure = 0;
 char debug = 0; /* XXX* */
 static int signalled = 0;
@@ -337,11 +339,17 @@ do_lockreq(msgctx_t *ctx, int req)
 static int
 dispatch_msg(msgctx_t *ctx, int nodeid, int need_close)
 {
-	int ret = 0, sz = -1, nid;
+	int ret = 0, sz = -1, nid, read_only = 1;
 	char msgbuf[4096];
 	generic_msg_hdr	*msg_hdr = (generic_msg_hdr *)msgbuf;
 	SmMessageSt	*msg_sm = (SmMessageSt *)msgbuf;
 
+	if (ctx->type == MSG_CLUSTER) {
+		read_only = 0;
+	} else if (ctx->u.local_info.cred.uid == 0) {
+		read_only = 0;
+	}
+
 	memset(msgbuf, 0, sizeof(msgbuf));
 
 	/* Peek-a-boo */
@@ -395,6 +403,10 @@ dispatch_msg(msgctx_t *ctx, int nodeid, int need_close)
 
 	case RG_LOCK:
 	case RG_UNLOCK:
+		if (read_only) {
+			msg_send_simple(ctx, RG_FAIL, RG_EPERM, 0);
+			goto out;
+		}
 		if (rg_quorate())
 			do_lockreq(ctx, msg_hdr->gh_command);
 		break;
@@ -407,6 +419,10 @@ dispatch_msg(msgctx_t *ctx, int nodeid, int need_close)
 		break;
 
 	case RG_ACTION_REQUEST:
+		if (read_only) {
+			msg_send_simple(ctx, RG_FAIL, RG_EPERM, 0);
+			goto out;
+		}
 
 		if (sz < (int)sizeof(msg_sm)) {
 			logt_print(LOG_ERR,
@@ -475,6 +491,11 @@ dispatch_msg(msgctx_t *ctx, int nodeid, int need_close)
 		return 0;
 
 	case RG_EVENT:
+		if (read_only) {
+			msg_send_simple(ctx, RG_FAIL, RG_EPERM, 0);
+			goto out;
+		}
+
 		/* Service event.  Run a dependency check */
 		if (sz < (int)sizeof(msg_sm)) {
 			logt_print(LOG_ERR,
@@ -497,6 +518,11 @@ dispatch_msg(msgctx_t *ctx, int nodeid, int need_close)
 		break;
 
 	case RG_EXITING:
+		if (read_only) {
+			msg_send_simple(ctx, RG_FAIL, RG_EPERM, 0);
+			goto out;
+		}
+
 		if (!member_online(msg_hdr->gh_arg1))
 			break;
 
@@ -507,6 +533,11 @@ dispatch_msg(msgctx_t *ctx, int nodeid, int need_close)
 		break;
 
 	case VF_MESSAGE:
+		if (read_only) {
+			msg_send_simple(ctx, RG_FAIL, RG_EPERM, 0);
+			goto out;
+		}
+
 		/* Ignore; our VF thread handles these
 		    - except for VF_CURRENT XXX (bad design) */
 		if (msg_hdr->gh_arg1 == VF_CURRENT)
@@ -514,6 +545,10 @@ dispatch_msg(msgctx_t *ctx, int nodeid, int need_close)
 		break;
 
 	default:
+		if (read_only) {
+			goto out;
+		}
+
 		logt_print(LOG_DEBUG, "unhandled message request %d\n",
 		       msg_hdr->gh_command);
 		break;
@@ -612,6 +647,8 @@ void dump_threads(FILE *fp);
 void dump_config_version(FILE *fp);
 void dump_vf_states(FILE *fp);
 void dump_cluster_ctx(FILE *fp);
+void dump_resource_info(FILE *fp);
+
 
 static void
 dump_internal_state(const char *loc)
@@ -625,6 +662,7 @@ dump_internal_state(const char *loc)
 	dump_thread_states(fp);
 #endif
 	dump_cluster_ctx(fp);
+	dump_resource_info(fp);
  	fclose(fp);
 }
 
@@ -943,6 +981,8 @@ main(int argc, char **argv)
 	}
 
 	init_logging(NULL, foreground, (debug? LOG_DEBUG : SYSLOGLEVEL));
+
+	rv = -1;
 	if (cman_connect(&clu) != 0)
 		goto out;	/* Clean exit if sigint/sigterm here */
 
@@ -953,6 +993,7 @@ main(int argc, char **argv)
 
 	if (clu_lock_init(rgmanager_lsname) != 0) {
 		printf("Locks not working!\n");
+		cman_finish(clu);
 		return -1;
 	}
 
@@ -962,14 +1003,16 @@ main(int argc, char **argv)
 	if (me.cn_nodeid == 0) {
 		printf("Unable to determine local node ID\n");
 		perror("cman_get_node");
-		return -1;
+		goto out_ls;
 	}
 	set_my_id(me.cn_nodeid);
 
 	logt_print(LOG_INFO, "I am node #%d\n", my_id());
 
-	if (wait_for_fencing() != 0)
-		goto out;
+	if (wait_for_fencing() != 0) {
+		rv = 0;
+		goto out_ls;
+	}
 
 	/*
 	   We know we're quorate.  At this point, we need to
@@ -980,24 +1023,26 @@ main(int argc, char **argv)
 
 	if (init_resource_groups(0, do_init) != 0) {
 		logt_print(LOG_CRIT, "#8: Couldn't initialize services\n");
-		return -1;
+		goto out_ls;
 	}
 
-	if (shutdown_pending)
-		goto out;
+	if (shutdown_pending) {
+		rv = 0;
+		goto out_ls;
+	}
 
 	if (msg_listen(MSG_SOCKET, RGMGR_SOCK, me.cn_nodeid, &local_ctx) < 0) {
 		logt_print(LOG_CRIT,
 		       "#10: Couldn't set up cluster message system: %s\n",
 		       strerror(errno));
-		return -1;
+		goto out_ls;
 	}
 
 	if (msg_listen(MSG_CLUSTER, &port, me.cn_nodeid, &cluster_ctx) < 0) {
 		logt_print(LOG_CRIT,
 		       "#10b: Couldn't set up cluster message system: %s\n",
 		       strerror(errno));
-		return -1;
+		goto out_ls;
 	}
 
 	rg_set_quorate();
@@ -1013,14 +1058,14 @@ main(int argc, char **argv)
 #ifdef OPENAIS
 	if (ds_init() < 0) {
 		logt_print(LOG_CRIT, "#11b: Couldn't initialize SAI AIS CKPT\n");
-		return -1;
+		goto out_ls;
 	}
 
 	ds_key_init("rg_lockdown", 32, 10);
 #else
 	if (vf_init(me.cn_nodeid, port, NULL, NULL) != 0) {
 		logt_print(LOG_CRIT, "#11: Couldn't set up VF listen socket\n");
-		return -1;
+		goto out_ls;
 	}
 
 	vf_key_init("rg_lockdown", 10, NULL, lock_commit_cb);
@@ -1045,6 +1090,8 @@ main(int argc, char **argv)
 
 	if (rg_initialized())
 		cleanup(cluster_ctx);
+	rv = 0;
+out_ls:
 	clu_lock_finished(rgmanager_lsname);
 
 out:
@@ -1055,5 +1102,5 @@ out:
 	/*malloc_stats();*/
 
 	daemon_cleanup();
-	exit(0);
+	exit(rv);
 }
