@@ -825,6 +825,11 @@ svc_migrate(const char *svcName, int target)
 		return RG_EINVAL;
 	}
 
+	if (m->cn_member == 0) {
+		free_member_list(membership);
+		return RG_ENODE;
+	}
+
 	count_resource_groups_local(m);
 	if (m->cn_svcexcl ||
 	    (m->cn_svccount && is_exclusive(svcName))) {
@@ -898,6 +903,9 @@ svc_migrate(const char *svcName, int target)
 		return RG_EFAIL;
 	case OCF_RA_NOT_CONFIGURED:
 		return RG_EINVAL;
+	case 150: /* see vm.sh */
+		/* Migration failed; VM still running on source node */
+		return RG_ERELO;
 	case 0:
 		break;
 	}
@@ -961,7 +969,7 @@ get_new_owner(const char *svcName)
 			continue;
 
 		if (msg_open(MSG_CLUSTER, membership->cml_members[x].cn_nodeid,
-			     RG_PORT, &ctx, 10) < 0) {
+			     RG_PORT, &ctx, 2 * cluster_timeout) < 0) {
 			/* failed to open: better to claim false successful
 			   status rather than claim a failure and possibly
 			   end up with a service on >1 node */
@@ -1246,7 +1254,10 @@ _svc_stop(const char *svcName, int req, int recover, uint32_t newstate)
 			rg_unlock(&lockp);
 			return RG_EFAIL;
 		}
-		/* FALLTHROUGH */
+		rg_unlock(&lockp);
+		broadcast_event(svcName, RG_STATE_STOPPED,
+				-1, svcStatus.rs_last_owner);
+		return RG_ESUCCESS;
 	case 2:
 		rg_unlock(&lockp);
 		return RG_ESUCCESS;
@@ -1545,7 +1556,7 @@ svc_start_remote(const char *svcName, int request, uint32_t target)
 	msg_relo.sm_data.d_svcOwner = target;
 	/* Open a connection to the other node */
 
-	if (msg_open(MSG_CLUSTER, target, RG_PORT, &ctx, 2)< 0) {
+	if (msg_open(MSG_CLUSTER, target, RG_PORT, &ctx, 2 * cluster_timeout)< 0) {
 		logt_print(LOG_ERR,
 		       "#58: Failed opening connection to member #%d\n",
 		       target);
@@ -1640,14 +1651,14 @@ svc_start_remote(const char *svcName, int request, uint32_t target)
  * @param new_owner	Member who actually ends up owning the service.
  */
 int
-handle_relocate_req(char *svcName, int request, int preferred_target,
+handle_relocate_req(char *svcName, int orig_request, int preferred_target,
 		    int *new_owner)
 {
 	cluster_member_list_t *allowed_nodes = NULL, *backup = NULL;
 	cman_node_t *m;
-	int target = preferred_target, me = my_id();
-	int ret, x;
 	rg_state_t svcStatus;
+	int target = preferred_target, me = my_id();
+	int ret, x, request = orig_request;
 	
 	get_rg_state_local(svcName, &svcStatus);
 	if (svcStatus.rs_state == RG_STATE_DISABLED ||
@@ -1749,6 +1760,13 @@ handle_relocate_req(char *svcName, int request, int preferred_target,
 				 */
 				return 0;
 			}
+
+			/*
+			 * Failed to start on that node.
+			 * Use the START_RECOVER operation on subsequent
+			 * attempts.
+			 */
+			request = RG_START_RECOVER;
 		}
 	}
 
@@ -1783,6 +1801,10 @@ handle_relocate_req(char *svcName, int request, int preferred_target,
 			return 0;
 		case RG_EDEPEND:
 		case RG_EFAIL:
+			/* Uh oh - we failed to relocate to this node.
+			   ensure that we tell the next node to start it from
+			   the 'recovering' state. */
+			request = RG_START_RECOVER;
 			memb_mark_down(allowed_nodes, target);
 			continue;
 		case RG_EABORT:
@@ -1815,7 +1837,7 @@ handle_relocate_req(char *svcName, int request, int preferred_target,
 	 * We got sent here from handle_start_req.
 	 * We're DONE.
 	 */
-	if (request == RG_START_RECOVER) {
+	if (orig_request == RG_START_RECOVER) {
 		_svc_stop_finish(svcName, 0, RG_STATE_STOPPED);
 		return RG_EFAIL;
 	}
@@ -1831,7 +1853,7 @@ exhausted:
 		       svcName);
 		if (svc_start(svcName, RG_START_RECOVER) == 0) {
 			*new_owner = me;
-			return RG_EFAIL;
+			return RG_ERELO;
 		}
 	}
 
@@ -2041,7 +2063,8 @@ handle_start_remote_req(char *svcName, int req)
 	if (need_check)
 		pthread_mutex_unlock(&exclusive_mutex);
 
-	if (svc_stop(svcName, RG_STOP_RECOVER) == 0)
+	if (svc_stop(svcName, central_events_enabled() ?
+		     RG_STATE_STOPPED : RG_STOP_RECOVER) == 0)
 		return RG_EFAIL;
 
 	svc_fail(svcName);
