@@ -155,6 +155,7 @@ ipv6_expand()
 	typeset addr=$1
 	typeset maskbits
 	typeset -i x
+	typeset tempaddr
 	
 	maskbits=${addr/*\//}
 	if [ "$maskbits" = "$addr" ]; then
@@ -164,6 +165,25 @@ ipv6_expand()
 		addr=${addr/\/*/}
 	fi
 
+	# grab each hex quad and expand it to 4 digits if it isn't already
+	# leave doublecolon in place for expansion out to the proper number of zeros later
+	tempaddr=""
+	for count in `seq 1 8`; do
+		quad=`echo $addr|awk -v count=$count -F : '{print $count}'`
+		quadlen=${#quad}
+		if [ $quadlen -eq 0 ]; then
+			quad=::
+		elif [ $quadlen -eq 1 ]; then
+			quad=000$quad
+		elif [ $quadlen -eq 2 ]; then
+			quad=00$quad
+		elif [ $quadlen -eq 3 ]; then
+			quad=0$quad
+		fi
+		tempaddr=$tempaddr$quad
+	done
+	addr=$tempaddr
+
 	# use space as placeholder
 	addr=${addr/::/\ }
 
@@ -408,46 +428,6 @@ ipv6_list_interfaces()
 }
 
 
-#
-# Find slaves for a bonded interface
-#
-findSlaves()
-{
-	declare mastif=$1
-	declare line
-	declare intf
-	declare interfaces
-
-	if [ -z "$mastif" ]; then
-		ocf_log err "usage: findSlaves <master I/F>"
-		return $OCF_ERR_ARGS
-	fi
-
-	line=$(/sbin/ip link list dev $mastif | grep "<.*MASTER.*>")
-	if [ $? -ne 0 ]; then
-		ocf_log err "Error determining status of $mastif"
-		return $OCF_ERR_GENERIC
-	fi
-
-	if [ -z "`/sbin/ip link list dev $mastif | grep \"<.*MASTER.*>\"`" ]
-	then
-		ocf_log err "$mastif is not a master device"
-		return $OCF_ERR_GENERIC
-	fi
-
-	## Strip possible VLAN (802.1q) suffixes 
-	##  - Roland Gadinger <roland.gadinger@beko.at> 
-	mastif=${mastif%%.*} 
-
-	while read line; do
-		set - $line
-		interfaces="${2/:/} $interfaces"
-	done < <( /sbin/ip -o link list | grep "master $mastif" )
-
-	echo $interfaces
-}
-
-
 isSlave()
 {
 	declare intf=$1
@@ -527,39 +507,8 @@ network_link_up()
 		return 1
 	fi
 	
-	#
-	# XXX assumes bond* interfaces are the bonding driver. (Fair
-	# assumption on Linux, I think)
-	#
-	if [ "${intf_arg/bond/}" != "$intf_arg" ]; then
-		
-		#
-		# With Xen we must check for slaves of pbondX, not bondX
-		#
-		masterif=$intf_arg
-		line=$(ip link list | awk {'print $2'} | grep "p$intf_arg:")
-		if [ $? -eq 0 ] ; then
-			masterif="p$intf_arg"
-		fi
-		
-		#
-		# Bonded driver.  Check link of all slaves for this interface.
-		# If any link is up, the bonding driver is expected to route
-		# traffic through that link.  Thus, the entire bonded link
-		# is declared up.
-		#
-		slaves=$(findSlaves $masterif)
-		if [ $? -ne 0 ]; then
-			ocf_log err "Error finding slaves of $intf_arg"
-			return 1
-		fi
-		for intf_test in $slaves; do
-			ethernet_link_up $intf_test && link_up=0
-		done
-	else
-		ethernet_link_up $intf_arg
-		link_up=$?
-	fi
+	ethernet_link_up $intf_arg
+	link_up=$?
 
 	if [ $link_up -eq 0 ]; then
 		ocf_log debug "Link for $intf_arg: Detected"
@@ -635,6 +584,13 @@ ipv6()
 			ocf_log info "Removing IPv6 address $addr from $dev"
                 fi
 		
+		if [ "$1" = "add" ]; then
+			ocf_log debug "Pinging addr ${addr%%/*} from dev $dev"
+			if ping_check inet6 ${addr%%/*} $dev; then
+				ocf_log err "IPv6 address collision ${addr%%/*}"
+				return 1
+			fi
+		fi
 		/sbin/ip -f inet6 addr $1 dev $dev $addr
 		[ $? -ne 0 ] && return 1
 		
@@ -707,6 +663,13 @@ ipv4()
 			ocf_log info "Removing IPv4 address $addr from $dev"
 		fi
 		
+		if [ "$1" = "add" ]; then
+			ocf_log debug "Pinging addr ${addr%%/*} from dev $dev"
+			if ping_check inet ${addr%%/*} $dev; then
+				ocf_log err "IPv4 address collision ${addr%%/*}"
+				return 1
+			fi
+		fi
 		/sbin/ip -f inet addr $1 dev $dev $addr
 		[ $? -ne 0 ] && return 1
 		
@@ -768,8 +731,22 @@ check_interface_up()
 {
 	declare dev
 	declare addr=${2/\/*/}
+	declare currentAddr caExpanded
+
+	if [ "$1" == "inet6" ]; then
+		addrExpanded=$(ipv6_expand $addr)
+		for currentAddr in `/sbin/ip -f $1 -o addr|awk '{print $4}'`; do
+			caExpanded=$(ipv6_expand $currentAddr)
+			caExpanded=${caExpanded/\/*/}
+			if [ "$addrExpanded" == "$caExpanded" ]; then
+				dev=$(/sbin/ip -f $1 -o addr | grep " ${currentAddr/\/*/}" | awk '{print $2}')
+				break
+			fi
+		done
+	else
+		dev=$(/sbin/ip -f $1 -o addr | grep " $addr/" | awk '{print $2}')
+	fi
 
-	dev=$(/sbin/ip -f $1 -o addr | grep " $addr/" | awk '{print $2}')
 	if [ -z "$dev" ]; then
 		return 1
 	fi
@@ -787,12 +764,26 @@ address_configured()
 {
 	declare line
 	declare addr
+	declare currentAddr caExpanded
 
 	# Chop off maxk bits 
 	addr=${2/\/*/}
-        line=$(/sbin/ip -f $1 -o addr | grep " $addr/")
 
-        if [ -z "$line" ]; then
+	if [ "$1" == "inet6" ]; then
+		addrExpanded=$(ipv6_expand $addr)
+		for currentAddr in `/sbin/ip -f $1 -o addr|awk '{print $4}'`; do
+			caExpanded=$(ipv6_expand $currentAddr)
+			caExpanded=${caExpanded/\/*/}
+			if [ "$addrExpanded" == "$caExpanded" ]; then
+				line=$(/sbin/ip -f $1 -o addr | grep " ${currentAddr/\/*/}");
+				break
+			fi
+		done
+	else
+		line=$(/sbin/ip -f $1 -o addr | grep " $addr/")
+	fi
+
+	if [ -z "$line" ]; then
 		return 1
 	fi
 	return 0
@@ -808,13 +799,26 @@ ip_op()
 	declare dev
 	declare rtr
 	declare addr=${3/\/*/}
-	
+	declare caExpanded currentAddr
 
 	if [ "$2" = "status" ]; then
 
 		ocf_log debug "Checking $3, Level $OCF_CHECK_LEVEL"
 	
-		dev=$(/sbin/ip -f $1 -o addr | grep " $addr/" | awk '{print $2}')
+		if [ "$1" == "inet6" ]; then
+			addrExpanded=$(ipv6_expand $addr)
+			for currentAddr in `/sbin/ip -f $1 -o addr|awk '{print $4}'`; do
+				caExpanded=$(ipv6_expand $currentAddr)
+				caExpanded=${caExpanded/\/*/}
+				if [ "$addrExpanded" == "$caExpanded" ]; then
+					dev=$(/sbin/ip -f $1 -o addr | grep " ${currentAddr/\/*/}" | awk '{print $2}')
+					break
+				fi
+			done
+		else
+			dev=$(/sbin/ip -f $1 -o addr | grep " $addr/" | awk '{print $2}')
+		fi
+
 		if [ -z "$dev" ]; then
 			ocf_log warn "$3 is not configured"
 			return 1
@@ -845,6 +849,19 @@ ip_op()
 		return $?
 		;;
 	inet6)
+		if [ "$2" = "del" ]; then
+			addrExpanded=$(ipv6_expand $addr)
+			for currentAddr in `/sbin/ip -f $1 -o addr|awk '{print $4}'`; do
+				caExpanded=$(ipv6_expand $currentAddr)
+				caExpanded=${caExpanded/\/*/}
+				if [ "$addrExpanded" == "$caExpanded" ]; then
+					addr6=$(/sbin/ip -f $1 -o addr | grep " ${currentAddr/\/*/}" | awk '{print $4}')
+					ipv6 $2 $addr6
+					return $?
+				fi
+			done
+		fi
+
 		ipv6 $2 $3
 		return $?
 		;;
